@@ -9,6 +9,7 @@ import numpy as np
         
 from matplotlib.lines import Line2D
 from matplotlib.patches import *
+from matplotlib import colors
 from .base import *
 from .logic import *
 from .logic.bitwise import *
@@ -23,30 +24,41 @@ netspacing = 10
 nettrackspacing = 10
 
 class MatplotlibRender:
-    def __init__(self, shape):
-        w = shape[0]
-        h = shape[1]
-        dpi = 100
-        iw = w / dpi 
-        ih = h / dpi
+    def __init__(self, shape, physical_shape=None, dpi=None):
+        w = shape[1]
+        h = shape[0]
+        if (physical_shape is None):
+            if (dpi is None):
+                dpi = 100
+            iw = w / dpi 
+            ih = h / dpi
+            #print('w/h = ',w,h, 'iw/ih', iw,ih)
+        else:
+            iw = physical_shape[1]
+            ih = physical_shape[0]
+            if (dpi is None):
+                dpi = max(w / iw, h / ih)
+            
         pmax = max(w,h)
         imax = max(iw, ih)
-        f = plt.figure(figsize=(imax,imax), dpi=dpi)
+        f = plt.figure(figsize=(iw,ih), dpi=dpi)
         self.canvas = f.add_subplot()
-        self.canvas.set_xlim(0, pmax)
-        self.canvas.set_ylim(0, pmax)
+        self.canvas.set_xlim(0, w)
+        self.canvas.set_ylim(0, h)
         self.canvas.invert_yaxis()
         plt.axis('off')
         
-        self.color = 'k'
-        self.fillcolor = 'k'
+        self.setForecolor('k')
+        self.setFillcolor('k')
         self.linewidth = 2
         
     def setForecolor(self, color):
-        self.color = color
+        #print('color {} = {}'.format(color, colors.rgb2hex(colors.to_rgb(color))))
+        self.color = colors.rgb2hex(colors.to_rgb(color))
         
     def setFillcolor(self, color):
-        self.fillcolor = color
+        #print('color {} = {}'.format(color, colors.rgb2hex(colors.to_rgb(color))))
+        self.fillcolor = colors.rgb2hex(colors.to_rgb(color))
         
     def setLineWidth(self, w):
         self.linewidth = w
@@ -101,8 +113,8 @@ class MatplotlibRender:
               boxstyle=BoxStyle("Round", pad=radius), facecolor=self.fillcolor)
         self.canvas.add_patch(box)
         
-    def drawArc(self, x0, y0, x1, y1, start, extent):
-        arc = Arc(((x0+x1)//2, (y0+y1)//2), x1-x0, y1-y0, angle=0, theta1=start, theta2=extent, linewidth=self.linewidth)
+    def drawArc(self, x0, y0, x1, y1, start, stop):
+        arc = Arc(((x0+x1)//2, (y0+y1)//2), x1-x0, y1-y0, angle=0, theta1=start, theta2=stop, linewidth=self.linewidth)
         self.canvas.add_patch(arc)
 
     def drawEllipse (self, x0, y0, x1, y1, outline=None, fill=None):
@@ -136,17 +148,19 @@ class TkinterRender:
         # self.canvas.invert_yaxis()
         # plt.axis('off')
         
-        self.color = 'k'
-        self.fillcolor = 'k'
+        self.setForecolor('k')
+        self.setFillcolor('k')
         self.linewidth = 2
         self.xmargin = 50
         self.ymargin = 50
         
     def setForecolor(self, color):
-        self.color = color
+        #print('color {} = {}'.format(color, colors.rgb2hex(colors.to_rgb(color))))
+        self.color = colors.rgb2hex(colors.to_rgb(color))
         
     def setFillcolor(self, color):
-        self.fillcolor = color
+        #print('color {} = {}'.format(color, colors.rgb2hex(colors.to_rgb(color))))
+        self.fillcolor = colors.rgb2hex(colors.to_rgb(color))
         
     def setLineWidth(self, w):
         self.linewidth = w
@@ -163,37 +177,40 @@ class TkinterRender:
         
     def drawPolygon(self, x, y, fill=False):
         
-        # points = []
-        
-        # for i in range(len(x)):
-        #     points.append(x[i])
-        #     points.append(y[i])
+        if (fill):
+            points = []
+            
+            for i in range(len(x)):
+                points.append(self.xmargin + x[i])
+                points.append(self.ymargin + y[i])
             
         # #lines = Line2D(x,y, color=self.color, linewidth=self.linewidth)
         # # if (fill):
         # #    self.canvas.fill(x, y, self.fillcolor)
 
-        # self.canvas.create_polygon(points)
-        x0 = x[0]
-        y0 = y[0]
-        for i in range(1, len(x)):
-            x1 = x[i]
-            y1 = y[i]
-            self.drawLine(x0 , y0 , x1 , y1 )
-            x0 = x1
-            y0 = y1
+            self.canvas.create_polygon(points, outline=self.color, fill=self.fillcolor, width=self.linewidth)
+            
+        else:            
+            x0 = x[0]
+            y0 = y[0]
+            for i in range(1, len(x)):
+                x1 = x[i]
+                y1 = y[i]
+                self.drawLine(x0 , y0 , x1 , y1 )
+                x0 = x1
+                y0 = y1
         #     points.append(x[i])
         #     points.append(y[i])
         
 
     def drawLine(self, x0, y0, x1, y1):
-        self.canvas.create_line(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin)
+        self.canvas.create_line(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin, fill=self.color, width=self.linewidth)
         #self.drawPolygon([x0, x1], [y0, y1])
         
     def drawRectangle(self , x0, y0, x1, y1, fill=False):
         #print('rect', x0, y0, x1, y1)
         #self.drawPolygon([x0, x1, x1, x0, x0],[y0,y0, y1,y1, y0], fill)
-        self.canvas.create_rectangle(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin)
+        self.canvas.create_rectangle(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin, width=self.linewidth)
         
     def drawRoundRectangle(self, x0, y0, x1, y1, radius=5, fill=False):
         #box = FancyBboxPatch((x0,y0), width=x1-x0, height=y1-y0,
@@ -201,24 +218,22 @@ class TkinterRender:
         #self.canvas.add_patch(box)
         print('[WARNING] round rectangle not supported yet')
         
-    def drawArc(self, x0, y0, x1, y1, start, extent):
+    def drawArc(self, x0, y0, x1, y1, start, stop):
         import tkinter
         # arc = Arc(((x0+x1)//2, (y0+y1)//2), x1-x0, y1-y0, angle=0, theta1=start, theta2=extent, linewidth=self.linewidth)
         # self.canvas.add_patch(arc)
-        print('draw arc', x0, y0, x1, y1, start, extent)
-        self.canvas.create_arc(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin, start=start, extent=extent-start, style=tkinter.ARC)
+        self.canvas.create_arc(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin, start=start, extent=stop-start, style=tkinter.ARC, width=self.linewidth)
 
     def drawEllipse (self, x0, y0, x1, y1, outline=None, fill=None):
         # el = Ellipse(((x0+x1)/2, (y0+y1)/2), x1-x0, y1-y0, edgecolor=self.color, facecolor='none', linewidth=self.linewidth)
         # self.canvas.add_artist(el)
-        self.canvas.create_oval(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin)
+        self.canvas.create_oval(x0 + self.xmargin, y0 + self.ymargin, x1 + self.xmargin, y1 + self.ymargin, width=self.linewidth)
         
     def drawSpline(self, x, y):
-        # from scipy import interpolate
-        # tck,u     = interpolate.splprep( [x,y] ,s = 0 )
-        # xnew,ynew = interpolate.splev( np.linspace( 0, 1, 20 ), tck,der = 0)
-        # self.canvas.plot( xnew ,ynew , color=self.color, linewidth=self.linewidth)
-        print('[WARNING] spline not supported yet')
+        from scipy import interpolate
+        tck,u     = interpolate.splprep( [x,y] ,s = 0 )
+        xnew,ynew = interpolate.splev( np.linspace( 0, 1, 20 ), tck,der = 0)
+        self.drawPolygon( xnew ,ynew)
 
         
 class Schematic:
@@ -228,7 +243,7 @@ class Schematic:
 
     step_timeout = 10 # any step of the schematic rendering cannot take more than this value (seconds)
     
-    def __init__(self, obj:Logic, render='matplotlib', parent=None):
+    def __init__(self, obj:Logic, render='matplotlib', parent=None, placeAndRoute=True):
    
         if not(obj.isStructural()):
             raise Exception('Schematics are only available to structural circuits')
@@ -259,6 +274,7 @@ class Schematic:
         
         self.mapping = {}
         self.mapping[And2] = AndSymbol
+        self.mapping[And] = AndSymbol
         self.mapping[Not] = NotSymbol
         self.mapping[Or2] = OrSymbol
         self.mapping[Or] = OrSymbol
@@ -277,8 +293,23 @@ class Schematic:
         self.mapping[Range] = RangeSymbol
 
         self.mapping[Waveform] = ScopeSymbol # Temp solution
-        
 
+        self.parent = parent
+        self.render = render        
+        self.canvas = None
+
+        if (placeAndRoute):
+            self.placeAndRoute()
+        
+        
+        # schematics are created but not directly drawn to allow 
+        # the manipulation of the graphical objects
+        
+        #self.drawAll()
+        
+        #mainloop()
+
+    def placeAndRoute(self):
         self.placeInputPorts()
         self.placeInstances()
         self.placeOutputPorts()
@@ -286,7 +317,7 @@ class Schematic:
         self.bruteForceSort()
         self.columnAssignment()
         
-        self.createNets()
+        self.createNets()                   
         self.passthroughCreation()
         self.removeArrowsSpecialCases()
         
@@ -307,24 +338,8 @@ class Schematic:
         self.routeNets()
         
         
-        if (render == 'matplotlib'):
-            self.canvas = MatplotlibRender(self.getOccupancyGrid().shape)
-        elif (render == 'tkinter'):
-            self.canvas = TkinterRender(parent, self.getOccupancyGrid().shape)
-        else:
-            raise Exception('Unsupported render {}'.format(render))
-            
-        self.canvas.setForecolor('k')
-        
-        # schematics are created but not directly drawn to allow 
-        # the manipulation of the graphical objects
-        
-        #self.drawAll()
-        
-        #mainloop()
-        
-        
     def draw(self):
+        self.drawAll()
         import matplotlib.pyplot as plt
         return plt.show()
     
@@ -359,32 +374,38 @@ class Schematic:
         self.channels = [] 
         
         #print('cols', self.columns)
-        for colidx in range(1, numCols):
+        for colidx in range(0, numCols):
             track = 0
-            netsInCol = [n for n in nets if n.sink in self.columns[colidx]]
+            netsInCol = [n for n in nets if n.sourcecol == colidx]
             
-            # temporal object to check if several nets are created from the same
-            # source
+            # temporal object to check if several nets are created from the 
+            # same source
             channeltracks = {}
             
             #print('Nets is column', colidx, netsInCol )
             for net in netsInCol:
                 try:
-                    existingtrack = channeltracks[net.source['wire']]
+                    existingtrack = channeltracks[net.wire]
+                    
                 except:
                     existingtrack = None
                     
+                
                 if (existingtrack == None):
+                    # There is not track for this wire
                     net.track = track
                     track = track + 1
                     channeltracks[net.wire]={'num':net.track, 'nets':[net]}
+                    #print('track:', net.track, 'channel tracks:', channeltracks)
                 else:
                     net.track = existingtrack['num']
                     existingtrack['nets'].append(net)
+                 
+                # do not update the sourcecol
+                #net.sourcecol = colidx - 1
                     
-                net.sourcecol = colidx - 1
-                    
-                
+            #print('col:', colidx, ['{}->{}'.format(type(x.source).__name__, type(x.sink).__name__)  for x in netsInCol])
+            #print('track:', track, 'channel tracks:', channeltracks)
             self.channels.append({'tracks':track, 'track':channeltracks})
         
     def getAllInstanceSources(self, sym:LogicSymbol):
@@ -452,9 +473,11 @@ class Schematic:
         for outp in outports:
             wire = outp.wire
             
-            for src in self.sinks:
-                if (src['wire'] == wire):
-                    ret.append(src['symbol'])
+            if not(wire in self.maxfanoutwires):
+                # avoid to report elements connected by pruned wires
+                for src in self.sinks:
+                    if (src['wire'] == wire):
+                        ret.append(src['symbol'])
         
         # if multiple wires connect between two symbols, there will be repetitions
         # so remove duplicates
@@ -482,12 +505,62 @@ class Schematic:
                 return colidx
             
         return -1
-                            
+                          
+    def columnManualAssignment(self, inlist, inslist, outlist):
+        self.columns = []
+        
+        
+        # Process inputs
+        unsorted = {}
+        currentCol = []
+        for idx, obj in enumerate(self.objs):
+            if (idx < self.numInputs):
+                unsorted[obj.obj.name] = obj
+                
+        for name in inlist:
+            currentCol.append(unsorted[name])
+        self.columns.append(currentCol)
+        
+        # Process instances
+        for inskeys in inslist:
+            unsorted = {}
+            currentCol = []
+            for idx, obj in enumerate(self.objs):
+                if (idx < self.numInputs):
+                    pass
+                elif (idx < self.firstOutput):
+                    if (obj.obj.name in inskeys):
+                        unsorted[obj.obj.name] = obj
+            
+            for name in inskeys:
+                currentCol.append(unsorted[name])
+            self.columns.append(currentCol)    
+        
+        # Process outputs
+        unsorted = {}
+        currentCol = []
+        for idx, obj in enumerate(self.objs):
+            if (idx >= self.firstOutput):
+                unsorted[obj.obj.name] = obj
+        for name in outlist:
+            currentCol.append(unsorted[name])
+        self.columns.append(currentCol)
+        
     def columnAssignment(self):
+        """
+        Creates the columns structure and assigns inputs, instances and outputs 
+        to a column
+
+        Returns
+        -------
+        None.
+
+        """
         self.columns = []
         currentCol = []
         
-        # append inputs
+        # append inputs. We know that inputs are always at the start
+        # of the object list
         for idx, obj in enumerate(self.objs):
             if (idx < self.numInputs):
                 currentCol.append(obj)
@@ -560,7 +633,7 @@ class Schematic:
             if (colidx < len(self.channels)):
                 self.channels[colidx]['sourcewidth'] = maxw
     
-    def passthroughCreation(self):
+    def passthroughCreation(self, debug=False):
         """
         Create passthrough entities.
         For connected entities that are separated by more than 1 column
@@ -590,12 +663,12 @@ class Schematic:
                     
                     if (sinkcol > colidx +1):
                         #print('WARNING: passthrough required between: [{}]'.format(colidx), type(obj).__name__, '[{}]'.format(sinkcol), type(sink).__name__, )
-                        self.insertPassthrough(obj, colidx, sink, sinkcol)
+                        self.insertPassthrough(obj, colidx, sink, sinkcol, debug)
                     if (sinkcol <= colidx):
                         #print('WARNING: feedback required between: [{}]'.format(colidx), type(obj).__name__, '[{}]'.format(sinkcol), type(sink).__name__)
-                        self.insertFeedback(obj, colidx, sink, sinkcol)
+                        self.insertFeedback(obj, colidx, sink, sinkcol, debug)
 
-    def insertPassthrough(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int):
+    def insertPassthrough(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int, debug=False):
         """
         A passthrough instance has to be inserted for every wire connecting both entities
 
@@ -636,6 +709,7 @@ class Schematic:
             
             for col in range(sourcecol+1, sinkcol):
                 pts = PassthroughSymbol()
+                pts.debug = debug
                 self.objs.append(pts)
                 self.columns[col].append(pts)
                 self.sources.append({'symbol':pts, 'wire':wire})
@@ -651,13 +725,14 @@ class Schematic:
                 
             if (lastSymbol != None):
                 net2 = NetSymbol(wire, pts, sink)
+                net2.sourcecol = sinkcol-1
                 self.nets.append(net2)
                                 
     def dumpNets(self):
         for idx, net in enumerate(self.nets):
             print('Net {} wire:{} source:{} {}  sink:{} {}'.format(idx, net.wire.getFullPath(), id(net.source), type(net.source).__name__ , id(net.sink), type(net.sink).__name__))
             
-    def insertFeedback(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int):
+    def insertFeedback(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int, debug=False):
         """
         A passthrough instance has to be inserted for every wire connecting both entities
 
@@ -701,33 +776,35 @@ class Schematic:
             
             #self.channels[sourcecol+1]
             fb_start = FeedbackStartSymbol()
+            fb_start.debug = debug
             self.objs.append(fb_start)
-            self.columns[sourcecol+1].append(fb_start)
+            self.columns[sourcecol].append(fb_start)
             #self.sources.append({'symbol':fb_start, 'wire':wire})
             #self.sinks.append({'symbol':fb_start, 'wire':wire})
             
             net1 = NetSymbol(wire, lastSymbol, fb_start)
             net1.sourcecol = sourcecol
             net1.arrow = False
-            self.nets.append(net1)
-            
+            self.nets.append(net1)            
             
             fb_end = FeedbackStopSymbol()
+            fb_end.debug = debug
             self.objs.append(fb_end)
-            self.columns[sinkcol-1].append(fb_end)
+            self.columns[sinkcol].append(fb_end)
             #self.sources.append({'symbol':fb_start, 'wire':wire})
             #self.sinks.append({'symbol':fb_start, 'wire':wire})
             
             net2 = NetSymbol(wire, fb_end, sink)
-            net2.sourcecol = sinkcol-1
+            net2.sourcecol = sinkcol
             net2.arrow = True
             self.nets.append(net2)
             
             lastSymbol = fb_end
             
             
-            for col in range(sinkcol, sourcecol+1):
+            for col in range(sinkcol+1, sourcecol):
                   pts = PassthroughSymbol()
+                  pts.debug = debug
                   self.objs.append(pts)
                   self.columns[col].append(pts)
                   self.sources.append({'symbol':pts, 'wire':wire})
@@ -743,6 +820,7 @@ class Schematic:
                 
             if (lastSymbol != None):
                  net2 = NetSymbol(wire, lastSymbol, fb_start)
+                 net2.sourcecol = sourcecol
                  net2.arrow = False
                  self.nets.append(net2)
 
@@ -946,6 +1024,28 @@ class Schematic:
         return ret(obj, x, y)
         
 
+    def placeInstance(self, child):
+        isym = self.getSymbol(child, self.x, self.y)
+        
+        if (isym is None):
+            isym = InstanceSymbol(child, self.x, self.y)
+        
+        self.objs.append(isym)
+        self.y = self.y + isym.getHeight() + cellmargin * 2 
+
+        i = 0
+        for inp in child.inPorts:
+            #print('adding inport ', child, inp.name)
+            self.sinks.append({'symbol':isym, 'x':0, 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
+            i = i+1
+        i = 0
+        for inp in child.outPorts:
+            #print('adding outport ', child, inp.name)
+            self.sources.append({'symbol':isym, 'x':isym.getWidth(), 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
+            i = i+1
+                
+        return isym
+    
     def placeInstances(self):
         maxx = 0
         self.y = gridsize*3
@@ -953,27 +1053,7 @@ class Schematic:
         #print('placeInstances', self.sys)
         
         for child in self.sys.children.values():
-            #print('child', child)
-            isym = self.getSymbol(child, self.x, self.y)
-            
-            if (isym is None):
-                isym = InstanceSymbol(child, self.x, self.y)
-            
-            self.objs.append(isym)
-            self.y = self.y + isym.getHeight() + cellmargin * 2 
- 
-            i = 0
-            for inp in child.inPorts:
-                #print('adding inport ', child, inp.name)
-                self.sinks.append({'symbol':isym, 'x':0, 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
-                i = i+1
-            i = 0
-            for inp in child.outPorts:
-                #print('adding outport ', child, inp.name)
-                self.sources.append({'symbol':isym, 'x':isym.getWidth(), 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
-                i = i+1
-                    
-    
+            isym = self.placeInstance(child)
             maxx = max(maxx, isym.getWidth())
             
         self.x = self.x + maxx + gridsize*10 + cellmargin
@@ -1015,7 +1095,7 @@ class Schematic:
     def createNets(self):
         """
         Create nets for all entities in the drawing.
-        It potulates the self.nets list by analyzing the sinks
+        It populates the self.nets list by analyzing the sinks
         list which was collected in function createOutputPorts
 
         Returns
@@ -1027,10 +1107,96 @@ class Schematic:
             try:
                 wire = sink['wire']
                 source = self.findSourceTuple(wire)
-                self.nets.append(NetSymbol(wire, source['symbol'], sink['symbol']))   
+                sourcecol = self.getSymbolColumn(source['symbol'])
+                sinkcol = self.getSymbolColumn(sink['symbol'])
+
+                net = NetSymbol(wire, source['symbol'], sink['symbol'])
+                net.sourcecol = sourcecol
+                net.sinkcol = sinkcol
+                self.nets.append(net)   
             except Exception as err:
                 print('Exception', err)
+                
+        self.maxfanoutwires = []
 
+    # def createNetsWithJoints(self):
+    #     """
+    #     Create nets for all entities in the drawing.
+    #     It populates the self.nets list by analyzing the sinks
+    #     list which was collected in function createOutputPorts
+
+    #     Returns
+    #     -------
+    #     None.
+
+    #     """
+        
+    #     # build a list for all the sinks of a wire
+    #     netSinks = {}
+    #     for src in self.sinks:
+    #         wire = src['wire']
+    #         try:
+    #             symlist = netSinks[wire]
+    #         except:
+    #             symlist = []
+    #         symlist.append(src['symbol'])
+    #         netSinks[wire] = symlist
+
+
+    #     for sink in self.sinks:
+    #         try:
+    #             wire = sink['wire']
+                
+    #             wireSinks = netSinks[wire]
+                
+    #             if (len(wireSinks) == 1):
+    #                 source = self.findSourceTuple(wire)
+    #                 self.nets.append(NetSymbol(wire, source['symbol'], sink['symbol']))   
+    #             else:
+                    
+    #         except Exception as err:
+    #             print('Exception', err)
+                
+    #     self.maxfanoutwires = []
+
+    def createNetsWithMaxFanout(self, maxfanout):
+        """
+        Create nets for all entities in the drawing.
+        It populates the self.nets list by analyzing the sinks
+        list which was collected in function createOutputPorts
+    
+        Returns
+        -------
+        None.
+    
+        """
+        sourceWires = {}
+        for src in self.sinks:
+            wire = src['wire']
+            try:
+                symlist = sourceWires[wire]
+            except:
+                symlist = []
+            symlist.append(src['symbol'])
+            sourceWires[wire] = symlist
+            
+        self.maxfanoutwires = [x for x in sourceWires.keys() if len(sourceWires[x]) > maxfanout] 
+            
+        for sink in self.sinks:
+            try:
+                wire = sink['wire']
+                
+                fanout = len(sourceWires[wire])
+                
+                if (fanout <= maxfanout):
+                    source = self.findSourceTuple(wire)
+                    self.nets.append(NetSymbol(wire, source['symbol'], sink['symbol']))   
+                else:
+                    print('skip wire {} fanout: {}'.format(wire.getFullPath(), fanout))
+                    
+            except Exception as err:
+                print('Exception', err)
+                
     def getNets(self):
         #oa = np.array(self.objs)
         #ba = np.array([isinstance(x, NetSymbol) for x in self.objs])
@@ -1042,8 +1208,42 @@ class Schematic:
         #ba = np.array([not isinstance(x, NetSymbol) for x in self.objs])
         #return list(oa[ba])
         return self.objs
-    
+
+    def createRender(self, physical_shape=None, dpi=None) :
+        """
+        Creates the renderer object
+
+        Parameters
+        ----------
+        physical_shape : tuple, optional
+            Dimension of the physical canvas (in inches). The default is None.
+        dpi : dots per inch, optional
+            Dots per inch of the drawing canvas. The default is None, and it is calculated.
+
+        Raises
+        ------
+        Exception
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
+        render = self.render
+        if (render == 'matplotlib'):
+            self.canvas = MatplotlibRender(self.getOccupancyGrid().shape, physical_shape, dpi)
+        elif (render == 'tkinter'):
+            self.canvas = TkinterRender(self.parent, self.getOccupancyGrid().shape)
+        else:
+            raise Exception('Unsupported render {}'.format(render))
+
     def drawAll(self):
+        if (self.canvas is None):
+            self.createRender()
+            
+        self.canvas.setForecolor('k')
+
         # Draw Instances
         
         # @todo the color and line properties should be assigned during creation 
@@ -1056,8 +1256,9 @@ class Schematic:
 
         # Draw Nets
         for obj in self.getNets():
-            self.canvas.setForecolor('blueviolet')  
-            self.canvas.setFillcolor('blueviolet')
+            
+            self.canvas.setForecolor(obj.color)  
+            self.canvas.setFillcolor(obj.color)
             self.canvas.setLineWidth(1)
             
             obj.draw(self.canvas)
@@ -1101,6 +1302,15 @@ class Schematic:
         self.objs[a], self.objs[b] = self.objs[b], self.objs[a] 
 
     def bruteForceSort(self):
+        """
+        Sort the object list
+
+        Returns
+        -------
+        am : TYPE
+            DESCRIPTION.
+
+        """
         am = self.getAdjacencyMatrix()
         nc = am.shape[0]
 
@@ -1257,7 +1467,21 @@ class Schematic:
         
         return v == 0
         
-    def routeNet(self, net:NetSymbol):
+    def routeNetSquare(self, net:NetSymbol):
+        """
+        Route a single net. The process consist in assigning 
+        a collection of points (path) to go from a source to a destination
+
+        Parameters
+        ----------
+        net : NetSymbol
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
         p0 = net.getStartPoint()
         pf = net.getEndPoint()
         
@@ -1278,32 +1502,114 @@ class Schematic:
                 print('source sym', net.source)
         
         if (hasattr(net, 'track') == False):
-            print('WARNING: net', net.source['wire'].getFullPath(), 'with not track')
+            print('WARNING: net', net.source, 'with not track')
             net.track = 0
 
-        mp = (net.source.x + sw + netspacing + net.track * nettrackspacing, (p0[1]+pf[1])//2)    
             
         if (isinstance(net.source, FeedbackStopSymbol)):
-            #print('Feedback-stop track:', net.track, mp[0])
-            net.setPath([mp[0], mp[0], pf[0]], [p0[1],pf[1],pf[1]])
+            # In stop symbols, the source and sink are in the same column
+            if (isinstance(net.sink, FeedbackStartSymbol) or isinstance(net.sink, PassthroughSymbol)):
+                # Normal case, midpoint is between both elements
+                mp = (net.source.x + sw + netspacing + net.track * nettrackspacing, (p0[1]+pf[1])//2)                
+                net.setPath([p0[0], mp[0], mp[0], pf[0]], [p0[1],p0[1],pf[1],pf[1]])
+            else:
+                mp = (net.source.x  - netspacing - net.track * nettrackspacing, -1)
+    
+                #print('Feedback-stop track:', net.track, mp[0])
+                #net.setPath([mp[0], mp[0], pf[0]], [p0[1],pf[1],pf[1]])
+                net.setPath([p0[0], mp[0], mp[0], pf[0]], [p0[1], p0[1], pf[1], pf[1]])
         elif (isinstance(net.sink, FeedbackStartSymbol)):
+            # In start symbols, the source and sink are in the same column, 
+            # midpoint is similar to normal case (in x)
+            mp = (net.source.x + sw + netspacing + net.track * nettrackspacing, -1)                
+            
             #print('Feedback-start track:', net.track, mp[0])
-            net.setPath([p0[0], mp[0], mp[0]], [p0[1],p0[1],pf[1]])
-        else:                  
+            net.setPath([p0[0], mp[0], mp[0], pf[0]], [p0[1], p0[1], pf[1], pf[1]])
+        else:                 
+            # Normal case, midpoint is between both elements
+            mp = (net.source.x + sw + netspacing + net.track * nettrackspacing, (p0[1]+pf[1])//2)                
             net.setPath([p0[0], mp[0], mp[0], pf[0]], [p0[1],p0[1],pf[1],pf[1]])
 
         #mp[0] = p0[0] + net.track * nettrackspacing
         
         net.routed = True
     
+    def routeNetDirect(self, net:NetSymbol):
+        """
+        Route a single net. The process consist in assigning 
+        a collection of points (path) to go from a source to a destination
+
+        Parameters
+        ----------
+        net : NetSymbol
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
+        p0 = net.getStartPoint()
+        pf = net.getEndPoint()
+        
+
+        # sw = 0
+        # try:
+        #     sw = self.channels[net.sourcecol]['sourcewidth'] 
+        #     #sw = sw - net.source['symbol'].getWidth() + netspacing
+        #     #print('sw[{}]={}'.format(net.sourcecol, sw))
+        # except:
+        #     if (hasattr(net, 'sourcecol') == False):
+        #         # this net was not assigned a sourcecol
+        #         print('WARNING: net', net.source['wire'].getFullPath(), 'without source column')
+        #         net.track = 0
+        #     else:
+        #         print('error sourcecol=', net.sourcecol)
+        #         print('channels[{}]'.format(net.sourcecol), self.channels[net.sourcecol])
+        #         print('source sym', net.source)
+        
+        # if (hasattr(net, 'track') == False):
+        #     print('WARNING: net', net.source['wire'].getFullPath(), 'with not track')
+        #     net.track = 0
+
+        # mp = (net.source.x + sw + netspacing + net.track * nettrackspacing, (p0[1]+pf[1])//2)    
+            
+        # if (isinstance(net.source, FeedbackStopSymbol)):
+        #     #print('Feedback-stop track:', net.track, mp[0])
+        #     net.setPath([mp[0], mp[0], pf[0]], [p0[1],pf[1],pf[1]])
+        # elif (isinstance(net.sink, FeedbackStartSymbol)):
+        #     #print('Feedback-start track:', net.track, mp[0])
+        #     net.setPath([p0[0], mp[0], mp[0]], [p0[1],p0[1],pf[1]])
+        # else:                  
+        #     net.setPath([p0[0], mp[0], mp[0], pf[0]], [p0[1],p0[1],pf[1],pf[1]])
+
+        net.setPath([p0[0], pf[0]], [p0[1],pf[1]])
+
+        #mp[0] = p0[0] + net.track * nettrackspacing
+        
+        net.routed = True
     
-    def routeNets(self):
+    def routeNets(self, mode='square'):
+        """
+        Route all the nets
+
+        Returns
+        -------
+        None.
+
+        """
         nets = self.getNets()
         
-        for net in nets:
-            # route net
-            self.routeNet(net)
-                        
+        if (mode == 'square'):
+            for net in nets:
+                # route net
+                self.routeNetSquare(net)
+        elif (mode == 'direct'):
+            for net in nets:
+                # route net
+                self.routeNetDirect(net)
+        else:
+            raise Exception('Unsupported net routing mode {}'.format(mode))
 
         # for inp in self.obj.outPorts:
         #     canvas.create_text(x+15-2, y, text=inp.name , anchor='e')
