@@ -132,7 +132,13 @@ class MulSymbol(BinaryOperatorSymbol):
 class AndSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
-        self.h = 40
+
+        if (isinstance(obj, And2)):
+            self.nins = 2
+        else:
+            self.nins = len(obj.ins)
+
+        self.h = 20 * self.nins
 
     def draw(self, canvas):
         x = self.x
@@ -145,7 +151,7 @@ class AndSymbol(LogicSymbol):
         canvas.drawLine(x, y, x + 25, y)
         canvas.drawLine(x, y + self.h, x + 25, y + self.h)
         canvas.drawLine(x, y, x, y + self.h)
-        canvas.drawArc(x , y+2, x + 50, y + self.h, start=-90, extent=90) #, style=tkinter.ARC, outline='black', fill='white')
+        canvas.drawArc(x , y+2, x + 50, y + self.h, start=-90, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
 
     def getHeight(self):
         return namemargin + self.h
@@ -165,12 +171,9 @@ class AndSymbol(LogicSymbol):
         if (selidx == -1):
             raise Exception('in port not found in {}'.format(self.obj.getFullPath()) )
 
-        if (selidx == 0):
-            y = self.h//2 - 10
-        else:
-            y = self.h//2 + 10 
+        y = 10 + selidx * 20
 
-        return (0, namemargin + y)
+        return (5, namemargin + y)
 
 class NotSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
@@ -347,7 +350,7 @@ class OrSymbol(LogicSymbol):
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x, y, x + 20, y)
         canvas.drawLine(x, y + self.h, x + 20, y + self.h)
-        canvas.drawArc(x-10 , y, x + 10, y + self.h, start=-90, extent=90) #, style=tkinter.ARC, outline='black', fill='white')
+        canvas.drawArc(x-10 , y, x + 10, y + self.h, start=-90, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
 
         #canvas.drawLine(x, y, x, y + self.h)
         #canvas.drawArc(x-30 , y, x + 50+20, y + self.h + (self.nins+1)*40, start=-90, extent=-60) #, style=tkinter.ARC, outline='black', fill='white')
@@ -392,11 +395,13 @@ class XorSymbol(LogicSymbol):
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x+10, y, x + 20+10, y)
         canvas.drawLine(x+10, y + self.h, x + 20+10, y + self.h)
-        canvas.drawArc(x-10 , y, x + 10, y + self.h, start=-90, extent=90) #, style=tkinter.ARC, outline='black', fill='white')
-        canvas.drawArc(x , y, x + 20, y + self.h, start=-90, extent=90) #, style=tkinter.ARC, outline='black', fill='white')
+        canvas.drawArc(x-10 , y, x + 10, y + self.h, start=-90, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
+        canvas.drawArc(x , y, x + 20, y + self.h, start=-90, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
 
-        canvas.drawArc(x-30 +10, y, x + 50+20+10, y + self.h*4, start=-90, extent=-60) #, style=tkinter.ARC, outline='black', fill='white')
-        canvas.drawArc(x-30 +10, y-self.h*3, x + 50+20+10, y + self.h, start=60, extent=90) #, style=tkinter.ARC, outline='black', fill='white')
+        #canvas.drawArc(x-30 +10, y, x + 50+20+10, y + self.h*4, start=-90, stop=-60) #, style=tkinter.ARC, outline='black', fill='white')
+        #canvas.drawArc(x-30 +10, y-self.h*3, x + 50+20+10, y + self.h, start=60, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
+        canvas.drawSpline([x+20+10, x+30+10, x+42+10, x+50+10], [y,  y+2, y+self.h//4, y+self.h//2])
+        canvas.drawSpline([x+20+10, x+30+10, x+42+10, x+50+10], [y+self.h,  y+self.h-2, y+self.h-self.h//4, y+self.h//2])
 
 
 
@@ -614,6 +619,7 @@ class Mux2Symbol(LogicSymbol):
 class PassthroughSymbol(LogicSymbol):
     def __init__(self):
         super().__init__(None, 0, 0)
+        self.debug = False
         
     def getHeight(self):
         return 30
@@ -624,15 +630,18 @@ class PassthroughSymbol(LogicSymbol):
     def draw(self, canvas):
         x = self.x
         y = self.y
-        
-        canvas.setForecolor('blueviolet')  
-        canvas.setLineWidth(1)
-        
-        #canvas.drawRectangle(x, y, x+30, y+30)
-        canvas.drawLine(x, y+15, x+30, y+15)
 
-        canvas.setForecolor('k')  
-        canvas.setLineWidth(2)
+        if (self.debug):       
+            canvas.setFillcolor('yellow')  
+            canvas.drawRectangle(x, y, x+30, y+30, fill=True)
+        else:
+            canvas.setForecolor('blueviolet')  
+            canvas.setLineWidth(1)
+            
+            canvas.drawLine(x, y+15, x+30, y+15)
+    
+            canvas.setForecolor('k')  
+            canvas.setLineWidth(2)
         
     def getWireSinkPos(self, wire:Wire):
         return (0, 15);
@@ -643,6 +652,7 @@ class PassthroughSymbol(LogicSymbol):
 class FeedbackStartSymbol(LogicSymbol):
     def __init__(self):
         super().__init__(None, 0, 0)
+        self.debug = False
         
     def getHeight(self):
         return 30
@@ -653,15 +663,19 @@ class FeedbackStartSymbol(LogicSymbol):
     def draw(self, canvas):
         x = self.x
         y = self.y
-        
-        canvas.setForecolor('red')  
-        canvas.setLineWidth(1)
-        
-        #canvas.drawRectangle(x, y, x+30, y+30)
-        canvas.drawLine(x, y+15, x+30, y+15)
 
-        canvas.setForecolor('k')  
-        canvas.setLineWidth(2)
+        if (self.debug):       
+            canvas.setFillcolor('lightgreen')  
+            canvas.drawRectangle(x, y, x+30, y+30, fill=True)
+        else:            
+            # canvas.setForecolor('red')  
+            # canvas.setLineWidth(1)
+            
+            # canvas.drawLine(x, y+15, x+30, y+15)
+        
+            # canvas.setForecolor('k')  
+            # canvas.setLineWidth(2)
+            pass
         
     def getWireSinkPos(self, wire:Wire):
         return (0, 15);
@@ -672,6 +686,7 @@ class FeedbackStartSymbol(LogicSymbol):
 class FeedbackStopSymbol(LogicSymbol):
     def __init__(self):
         super().__init__(None, 0, 0)
+        self.debug = False
         
     def getHeight(self):
         return 30
@@ -682,15 +697,20 @@ class FeedbackStopSymbol(LogicSymbol):
     def draw(self, canvas):
         x = self.x
         y = self.y
-        
-        canvas.setForecolor('red')  
-        canvas.setLineWidth(1)
-        
-        #canvas.drawRectangle(x, y, x+30, y+30)
-        canvas.drawLine(x, y+15, x+30, y+15)
 
-        canvas.setForecolor('k')  
-        canvas.setLineWidth(2)
+        if (self.debug):       
+            canvas.setFillcolor('red')  
+            canvas.drawRectangle(x, y, x+30, y+30, fill=True)
+        else:
+            # canvas.setForecolor('red')  
+            # canvas.setLineWidth(1)
+            
+            # #canvas.drawRectangle(x, y, x+30, y+30)
+            # canvas.drawLine(x, y+15, x+30, y+15)
+    
+            # canvas.setForecolor('k')  
+            # canvas.setLineWidth(2)
+            pass
         
     def getWireSinkPos(self, wire:Wire):
         return (0, 15);
@@ -702,7 +722,7 @@ class FeedbackStopSymbol(LogicSymbol):
 class NetSymbol:
     def __init__(self, wire:Wire, source:LogicSymbol, sink:LogicSymbol):
         """
-        Constructor of net symbol
+        A net symbol connects two entities.
 
         Parameters
         ----------
@@ -725,6 +745,9 @@ class NetSymbol:
         self.y = None
         self.routed = False
         self.arrow = True
+        self.color = 'blueviolet'
+        self.sourcecol = -1
+        # self.sinkcol will be assigned later
         
     def getStartPoint(self):
         objsource = self.source
